local addon = LibStub("AceAddon-3.0"):GetAddon("Ouro Loot",--[[silent=]]true)
if not addon then return end
local csv = addon:GetModule("CSV",--[[silent=]]true)
if not csv then return end

local pairs, ipairs, tostring, tonumber = pairs, ipairs, tostring, tonumber
local tconcat, wipe = table.concat, table.wipe

local SKIP = csv.skiptoken

-----------------------------------------------------------------------------
--[===[    LOOT

Callbacks follow simple rules:

-  Each row of the main Loot window corresponds to an "entry".  Entries have
   different types:  time, boss, or loot.

-  Every method is passed a table ("e") containing information about that
   entry.  Some methods may get additional parameters; if they are not needed
   they can be ignored or even left out of the parameter list.

-  Every method must return either a string to be used in the text, or SKIP
   to ignore that entry and move on to the next row.

-  Not returning anything is the same as returning an empty string.  This
   results in an empty field/column within that line.

-  Method names are entirely lowercase, no matter how the field names in
   the editor are written.

For 'time' entries, a single 'time' method is called.  This isn't meant to
generate any text, but can make note of the timestamp for later.  If no such
method is defined, the entry is skipped.

For 'boss' entries, a reason-specific method "boss_<reason>" is called.  The
reasons are determined by DBM, but to date only "wipe" and "kill" have ever
been used.  If no such method is defined, the entry is skipped.

For 'loot' entries, one method for each field in the format string is called,
"loot_<field>".  If no such method is defined, that field in the output line
will be empty, but the line as a whole can still be produced as long as none
of the other field methods return SKIP.

A reference to the contents of entry tables is given at the top of the
Ouro_Loot/core.lua file (look for "g_loot indicies").
]===]


local last_time_entry
function csv:time (e)
	last_time_entry = e
	return SKIP
end

function csv:boss_kill (e)
	-- could record stuff for later, like time does
	return SKIP
end

function csv:loot_itemname (e)
	return e.itemname
end

function csv:loot_playername (e)
	return e.person
end

function csv:loot_disposition (e)
	return e.disposition or 'mainspec'  -- 'gvault', 'offspec', 'shard'
end

function csv:loot_datestamp (e)
	if last_time_entry == nil then
		return _G.TIME_UNKNOWN
	end
	if self:use_basic_day_p('loot') then
		return last_time_entry.startday.text  -- "dd Month yyyy"
	else
		return tostring (date (self.opts.date_loot, e.stamp))
	end
end

function csv:loot_itemcount (e)
	-- Count text is usually prefixed with an x.
	return e.count and (e.count:gsub("x","")) or "1"
end

function csv:loot_extra (e)
	return e.extratext_byhand and e.extratext
end

--[[
-- E:      would be a typical loot entry
-- FIELD:  would be "Example", currently redundant and unused
function csv:loot_example (e, field)
end
]]


-----------------------------------------------------------------------------
--[===[    RAIDERS ATTENDANCE

Unlike the loot callbacks, these are only done for 'time' and 'boss' events.
For 'time' entries, the same method as above is called.  For 'boss' entries,
we iterate over the snapshot of the raid roster taken at that time.

Callback names are "raiders_<field>", where the field names are taken from
the format string edited in-game.  The return/skip rules are the same.  The
arguments are the tag name (normally unused), the player name, and the player
info table, also described with the loot entry tables above.
]===]


local max_group_number
function csv:raiders_boss (e, field, player, info)
	-- If the fight was recorded as less than 10 seconds, assume it was
	-- either a mispull (not interesting) or a data error.  This might be
	-- a dumb idea; somebody sitting outside the instance recording loot
	-- will only see zero-duration fights.
	if e.duration < 10 then
		return SKIP
	end
	-- Assume that every format string will use "Boss" somewhere, and use
	-- it to cache this number.
	max_group_number = e.maxsize / MEMBERS_PER_RAID_GROUP

	return e.bossname
end

function csv:raiders_datestamp (e, field, player, info)
	if last_time_entry == nil then
		return _G.TIME_UNKNOWN
	end
	if self:use_basic_day_p('raiders') then
		return last_time_entry.startday.text  -- "dd Month yyyy"
	else
		return tostring (date (self.opts.date_raiders, e.stamp))
	end
end

function csv:raiders_playername (e, field, player, info)
	return player
end

function csv:raiders_status (e, field, player, info)
	if (not info.online) or (info.online == 3) then
		-- no longer in raid group
		return SKIP
	end
	if info.subgroup <= max_group_number then
		if info.online == 2 then
			-- in the raid groups, but offline
			-- (this might not be recorded)
			return [[Offline]]
		else
			return [[Fighting]]
		end
	else
		return [[Sitting Out]]
	end
end

-- "Joined" can also mean "first time we've seen them", i.e., we join an
-- existing raid, everybody will have "joined" at that moment.
--
-- "Left" can also mean the time that we leave the raid, i.e., when we bail,
-- everybody behind us will have "left" at that moment, but their online
-- status will not have changed.
--
-- The way to tell is to look for multiple people all changing state at the
-- same (recorded) time, but that really needs a human's eyes.
--
-- This code could be compressed considerably, but becomes a bit unreadable.
function csv:raiders_joined (e, field, player, info)
	if self:use_basic_day_p('raiders') then
		if last_time_entry == nil then
			return _G.TIME_UNKNOWN
		end
		return last_time_entry.startday.text  -- "dd Month yyyy"
	else
		if field == 'joined' then
			return tostring (date (self.opts.date_raiders, info.join))
		elseif field == 'left' then
			return tostring (date (self.opts.date_raiders, info.leave))
		end
	end
end
csv.raiders_left = csv.raiders_joined
--function csv:raiders_left (e, field, player, info)
--
--end


-- vim:noet
